# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, List, cast

import pytest

from giskard_hub import HubClient, AsyncHubClient
from tests.utils import assert_matches_type
from giskard_hub.types import TestCase

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTestCases:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: HubClient) -> None:
        test_case = client.test_cases.create(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: HubClient) -> None:
        test_case = client.test_cases.create(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            checks=[
                {
                    "identifier": "correctness",
                    "params": {
                        "reference": "reference",
                    },
                    "enabled": True,
                }
            ],
            demo_output="content",
            status="active",
            tags=["string"],
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: HubClient) -> None:
        response = client.test_cases.with_raw_response.create(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: HubClient) -> None:
        with client.test_cases.with_streaming_response.create(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: HubClient) -> None:
        test_case = client.test_cases.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: HubClient) -> None:
        response = client.test_cases.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: HubClient) -> None:
        with client.test_cases.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `test_case_id` but received ''",
        ):
            client.test_cases.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: HubClient) -> None:
        test_case = client.test_cases.update(
            test_case_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: HubClient) -> None:
        test_case = client.test_cases.update(
            test_case_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            checks=[
                {
                    "identifier": "correctness",
                    "params": {
                        "reference": "reference",
                    },
                    "enabled": True,
                }
            ],
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            demo_output="content",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            tags=["string"],
            status="active",
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: HubClient) -> None:
        response = client.test_cases.with_raw_response.update(
            test_case_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: HubClient) -> None:
        with client.test_cases.with_streaming_response.update(
            test_case_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `test_case_id` but received ''",
        ):
            client.test_cases.with_raw_response.update(
                test_case_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: HubClient) -> None:
        test_case = client.test_cases.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: HubClient) -> None:
        response = client.test_cases.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: HubClient) -> None:
        with client.test_cases.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(None, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: HubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `test_case_id` but received ''",
        ):
            client.test_cases.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_delete(self, client: HubClient) -> None:
        test_case = client.test_cases.bulk_delete(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_bulk_delete(self, client: HubClient) -> None:
        response = client.test_cases.with_raw_response.bulk_delete(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_bulk_delete(self, client: HubClient) -> None:
        with client.test_cases.with_streaming_response.bulk_delete(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(None, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_update(self, client: HubClient) -> None:
        test_case = client.test_cases.bulk_update(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(List[TestCase], test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_update_with_all_params(self, client: HubClient) -> None:
        test_case = client.test_cases.bulk_update(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            disabled_checks=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            enabled_checks=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(List[TestCase], test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_bulk_update(self, client: HubClient) -> None:
        response = client.test_cases.with_raw_response.bulk_update(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(List[TestCase], test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_bulk_update(self, client: HubClient) -> None:
        with client.test_cases.with_streaming_response.bulk_update(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(List[TestCase], test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_move(self, client: HubClient) -> None:
        test_case = client.test_cases.bulk_move(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            target_dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_bulk_move_with_all_params(self, client: HubClient) -> None:
        test_case = client.test_cases.bulk_move(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            target_dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            duplicate=True,
        )
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_bulk_move(self, client: HubClient) -> None:
        response = client.test_cases.with_raw_response.bulk_move(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            target_dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = response.parse()
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_bulk_move(self, client: HubClient) -> None:
        with client.test_cases.with_streaming_response.bulk_move(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            target_dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = response.parse()
            assert_matches_type(None, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTestCases:
    parametrize = pytest.mark.parametrize(
        "async_client",
        [False, True, {"http_client": "aiohttp"}],
        indirect=True,
        ids=["loose", "strict", "aiohttp"],
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.create(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.create(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            checks=[
                {
                    "identifier": "correctness",
                    "params": {
                        "reference": "reference",
                    },
                    "enabled": True,
                }
            ],
            demo_output="content",
            status="active",
            tags=["string"],
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncHubClient) -> None:
        response = await async_client.test_cases.with_raw_response.create(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncHubClient) -> None:
        async with async_client.test_cases.with_streaming_response.create(
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncHubClient) -> None:
        response = await async_client.test_cases.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncHubClient) -> None:
        async with async_client.test_cases.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `test_case_id` but received ''",
        ):
            await async_client.test_cases.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.update(
            test_case_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.update(
            test_case_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            checks=[
                {
                    "identifier": "correctness",
                    "params": {
                        "reference": "reference",
                    },
                    "enabled": True,
                }
            ],
            dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            demo_output="content",
            messages=[
                {
                    "content": "content",
                    "role": "role",
                }
            ],
            tags=["string"],
        )
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncHubClient) -> None:
        response = await async_client.test_cases.with_raw_response.update(
            test_case_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(TestCase, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncHubClient) -> None:
        async with async_client.test_cases.with_streaming_response.update(
            test_case_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(TestCase, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `test_case_id` but received ''",
        ):
            await async_client.test_cases.with_raw_response.update(
                test_case_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncHubClient) -> None:
        response = await async_client.test_cases.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncHubClient) -> None:
        async with async_client.test_cases.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(None, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncHubClient) -> None:
        with pytest.raises(
            ValueError,
            match=r"Expected a non-empty value for `test_case_id` but received ''",
        ):
            await async_client.test_cases.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_delete(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.bulk_delete(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_bulk_delete(self, async_client: AsyncHubClient) -> None:
        response = await async_client.test_cases.with_raw_response.bulk_delete(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_delete(self, async_client: AsyncHubClient) -> None:
        async with async_client.test_cases.with_streaming_response.bulk_delete(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(None, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_update(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.bulk_update(
            ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(List[TestCase], test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_update_with_all_params(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.bulk_update(
            ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            disabled_checks=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            enabled_checks=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )
        assert_matches_type(List[TestCase], test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_bulk_update(self, async_client: AsyncHubClient) -> None:
        response = await async_client.test_cases.with_raw_response.bulk_update(
            ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(List[TestCase], test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_update(self, async_client: AsyncHubClient) -> None:
        async with async_client.test_cases.with_streaming_response.bulk_update(
            ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(List[TestCase], test_case, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_move(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.bulk_move(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            target_dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_bulk_move_with_all_params(self, async_client: AsyncHubClient) -> None:
        test_case = await async_client.test_cases.bulk_move(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            target_dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            duplicate=True,
        )
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_bulk_move(self, async_client: AsyncHubClient) -> None:
        response = await async_client.test_cases.with_raw_response.bulk_move(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            target_dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Giskard-Lang") == "python"
        test_case = await response.parse()
        assert_matches_type(None, test_case, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_bulk_move(self, async_client: AsyncHubClient) -> None:
        async with async_client.test_cases.with_streaming_response.bulk_move(
            test_case_ids=["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
            target_dataset_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Giskard-Lang") == "python"

            test_case = await response.parse()
            assert_matches_type(None, test_case, path=["response"])

        assert cast(Any, response.is_closed) is True



"""Common type definitions shared across resources."""

from typing import Any, Dict, List, Union, Generic, Literal, TypeVar, Optional, TypeAlias
from typing_extensions import Required, TypedDict, override

from ..._models import BaseModel

__all__ = [
    "APIResponse",
    "APIPaginatedMetadata",
    "APIPaginatedResponse",
    "APIResponseWithIncluded",
    "PaginatedMetadata",
    "FilterValueParam",
    "ListFilterValueParam",
    "DateRangeFilterValueParam",
    "OrderByParam",
    "TaskState",
    "TaskProgress",
    "TaskProgressParam",
]

T = TypeVar("T")
TIncluded = TypeVar("TIncluded")


# ---------------------------------------------------------------------------
# Response wrappers
# ---------------------------------------------------------------------------


class APIResponse(BaseModel, Generic[T]):
    data: T

    @override
    def __repr__(self) -> str:
        return f"APIResponse[{type(self.data).__name__}](data={self.data!r})"


class APIResponseWithIncluded(BaseModel, Generic[T, TIncluded]):
    data: T
    included: Optional[Dict[str, Dict[str, TIncluded]]] = None


class APIPaginatedMetadata(BaseModel):
    count: int
    offset: int
    limit: int
    total: int


class APIPaginatedResponse(BaseModel, Generic[T, TIncluded]):
    data: List[T]
    included: Optional[Dict[str, Dict[str, TIncluded]]] = None
    metadata: APIPaginatedMetadata


class PaginatedMetadata(BaseModel):
    total: int
    limit: int
    offset: int


# ---------------------------------------------------------------------------
# Filter / ordering helpers
# ---------------------------------------------------------------------------


class ListFilterValueParam(TypedDict, total=False):
    selected_options: List[Any]
    is_empty: bool
    match_mode: Literal["include", "exclude"]
    match_logic: Literal["any", "all"]


class DateRangeFilterValueParam(TypedDict, total=False):
    from_: Optional[str]
    to_: Optional[str]


FilterValueParam: TypeAlias = Union[ListFilterValueParam, DateRangeFilterValueParam]


class OrderByParam(TypedDict, Generic[T], total=False):
    id: Required[T]
    desc: bool


# ---------------------------------------------------------------------------
# Task state / progress (shared across many domain modules)
# ---------------------------------------------------------------------------

TaskState: TypeAlias = Literal["skipped", "finished", "error", "running", "canceled"]


class TaskProgress(BaseModel):
    state: TaskState
    current: int
    total: int
    error: Optional[str] = None


class TaskProgressParam(TypedDict, total=False):
    state: TaskState
    current: int
    total: int
    error: Optional[str]



from giskard_hub import HubClient, AsyncHubClient
from giskard_hub._client import _normalize_base_url

API_KEY = "test-key"


class TestNormalizeBaseUrl:
    def test_appends_suffix_when_missing(self) -> None:
        assert _normalize_base_url("http://localhost:8080", auto_add_api_suffix=True) == "http://localhost:8080/_api"

    def test_does_not_double_append_suffix(self) -> None:
        assert (
            _normalize_base_url("http://localhost:8080/_api", auto_add_api_suffix=True) == "http://localhost:8080/_api"
        )

    def test_strips_trailing_slash_before_appending(self) -> None:
        assert _normalize_base_url("http://localhost:8080/", auto_add_api_suffix=True) == "http://localhost:8080/_api"

    def test_strips_trailing_slash_without_appending(self) -> None:
        assert _normalize_base_url("http://localhost:8080/", auto_add_api_suffix=False) == "http://localhost:8080"

    def test_disabled_does_not_append_suffix(self) -> None:
        assert _normalize_base_url("http://localhost:8080", auto_add_api_suffix=False) == "http://localhost:8080"


class TestHubClientBaseUrl:
    def test_suffix_added_by_default(self) -> None:
        client = HubClient(base_url="http://localhost:8080", api_key=API_KEY)
        assert str(client.base_url).rstrip("/") == "http://localhost:8080/_api"

    def test_suffix_not_duplicated(self) -> None:
        client = HubClient(base_url="http://localhost:8080/_api", api_key=API_KEY)
        assert str(client.base_url).rstrip("/") == "http://localhost:8080/_api"

    def test_suffix_not_added_when_disabled(self) -> None:
        client = HubClient(base_url="http://localhost:8080", api_key=API_KEY, auto_add_api_suffix=False)
        assert str(client.base_url).rstrip("/") == "http://localhost:8080"

    def test_trailing_slash_stripped(self) -> None:
        client = HubClient(base_url="http://localhost:8080/", api_key=API_KEY)
        assert str(client.base_url).rstrip("/") == "http://localhost:8080/_api"


class TestAsyncHubClientBaseUrl:
    def test_suffix_added_by_default(self) -> None:
        client = AsyncHubClient(base_url="http://localhost:8080", api_key=API_KEY)
        assert str(client.base_url).rstrip("/") == "http://localhost:8080/_api"

    def test_suffix_not_duplicated(self) -> None:
        client = AsyncHubClient(base_url="http://localhost:8080/_api", api_key=API_KEY)
        assert str(client.base_url).rstrip("/") == "http://localhost:8080/_api"

    def test_suffix_not_added_when_disabled(self) -> None:
        client = AsyncHubClient(base_url="http://localhost:8080", api_key=API_KEY, auto_add_api_suffix=False)
        assert str(client.base_url).rstrip("/") == "http://localhost:8080"

    def test_trailing_slash_stripped(self) -> None:
        client = AsyncHubClient(base_url="http://localhost:8080/", api_key=API_KEY)
        assert str(client.base_url).rstrip("/") == "http://localhost:8080/_api"



from __future__ import annotations

from typing import List, Literal, Iterable, Optional, cast

import httpx

from ...types import Agent, Dataset
from .results import (
    ResultsResource,
    AsyncResultsResource,
    ResultsResourceWithRawResponse,
    AsyncResultsResourceWithRawResponse,
    ResultsResourceWithStreamingResponse,
    AsyncResultsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .._included import embed_included_list, embed_included_single
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.chat import ChatMessageParam
from ...types.agent import AgentOutputParam, MinimalAgentParam
from ...types.check import CheckResult, CheckConfigParam
from ..._base_client import make_request_options
from ...types.common import APIResponse, APIResponseWithIncluded
from ...types.dataset import DatasetSubsetParam
from ...types.evaluation import (
    Evaluation,
    EvaluationListParams,
    EvaluationCreateParams,
    EvaluationUpdateParams,
    EvaluationRetrieveParams,
    EvaluationRunSingleParams,
    CriterionEvaluationDataset,
    EvaluationBulkDeleteParams,
    EvaluationCreateLocalParams,
)

__all__ = ["EvaluationsResource", "AsyncEvaluationsResource"]


def _validate_dataset_or_old_evaluation(
    dataset_id: Optional[str] | Omit,
    old_evaluation_id: Optional[str] | Omit,
) -> None:
    """
    Ensure that exactly one of `dataset_id` or `old_evaluation_id` is provided.

    Raises
    ------
    ValueError
        If both are provided or both are omitted.
    """
    if (dataset_id is omit and old_evaluation_id is omit) or (dataset_id is not omit and old_evaluation_id is not omit):
        raise ValueError("Exactly one of `dataset_id` or `old_evaluation_id` must be provided")


def _check_params_to_api(checks: Iterable[CheckConfigParam]) -> Iterable[dict[str, object]]:
    return [{"identifier": check["identifier"], **(check.get("params", {}))} for check in checks]


class EvaluationsResource(SyncAPIResource):
    @cached_property
    def results(self) -> ResultsResource:
        return ResultsResource(self._client)

    @cached_property
    def with_raw_response(self) -> EvaluationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return EvaluationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EvaluationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return EvaluationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        project_id: str,
        agent_id: str,
        name: str | Omit = omit,
        dataset_id: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        old_evaluation_id: Optional[str] | Omit = omit,
        run_count: int | Omit = omit,
        scheduled_evaluation_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Evaluation:
        """Create and launch a new evaluation of an agent on a dataset.

        Parameters
        ----------
        project_id : str
            The ID of the project to create the evaluation for.
        agent_id : str
            The ID of the agent to create the evaluation for.
        name : str or Omit
            The name of the evaluation.
        dataset_id : str, optional
            The ID of the dataset to draw test cases from. Exactly one of
            ``dataset_id`` or ``old_evaluation_id`` must be provided.
        tags : sequence of str, optional
            Optional tags to restrict the subset to test cases carrying those
            tags. Only used when ``dataset_id`` is provided.
        old_evaluation_id : str, optional
            The ID of a previous evaluation whose test cases should be reused.
            Exactly one of ``old_evaluation_id`` or ``dataset_id`` must be
            provided.
        run_count : int or Omit
            The number of times to run each test case.
        scheduled_evaluation_id : str, optional
            The ID of the scheduled evaluation to associate with.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        Evaluation
            The newly created evaluation.

        Raises
        ------
        ValueError
            If neither or both of ``dataset_id`` and ``old_evaluation_id`` are
            provided.
        """

        _validate_dataset_or_old_evaluation(dataset_id, old_evaluation_id)
        criteria: DatasetSubsetParam | CriterionEvaluationDataset | Omit = omit

        if dataset_id is not omit:
            criteria = DatasetSubsetParam(
                dataset_id=cast(str, dataset_id),
                tags=None if tags is omit else cast(Optional[SequenceNotStr[str]], tags),
            )

        response = self._post(
            "/v2/evaluations",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "project_id": project_id,
                    "criteria": criteria,
                    "name": name,
                    "old_evaluation_id": old_evaluation_id,
                    "run_count": run_count,
                    "scheduled_evaluation_id": scheduled_evaluation_id,
                },
                EvaluationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Evaluation],
        )

        return self._unwrap(response)

    def retrieve(
        self,
        evaluation_id: str,
        *,
        include: Optional[List[Literal["agent", "dataset"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Evaluation:
        """Retrieve an evaluation by its ID, with optional related resource inclusion.

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to retrieve.
        include : list of {"agent", "dataset"}, optional
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        Evaluation
            The requested evaluation.

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")

        response = self._get(
            f"/v2/evaluations/{evaluation_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include": include}, EvaluationRetrieveParams),
            ),
            cast_to=APIResponseWithIncluded[Evaluation, APIResponse[Agent | Dataset]],
        )

        if include is not omit and include:
            response = embed_included_single(response, id_getter=lambda evaluation: evaluation.id)

        return self._unwrap(response)

    def update(
        self,
        evaluation_id: str,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Evaluation:
        """Update an evaluation's name.

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to update.
        name : str
            The name of the evaluation.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        Evaluation
            The updated evaluation.

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        response = self._patch(
            f"/v2/evaluations/{evaluation_id}",
            body=maybe_transform({"name": name}, EvaluationUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Evaluation],
        )

        return self._unwrap(response)

    def list(
        self,
        *,
        project_id: str,
        include: Optional[List[Literal["agent", "dataset"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Evaluation]:
        """List all evaluations for a project, with optional related resource inclusion.

        Parameters
        ----------
        project_id : str
            The ID of the project to list evaluations for.
        include : list of {"agent", "dataset"}, optional
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        list of Evaluation
            The evaluations belonging to the project.
        """
        response = self._get(
            "/v2/evaluations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "project_id": project_id,
                        "include": include,
                    },
                    EvaluationListParams,
                ),
            ),
            cast_to=APIResponseWithIncluded[List[Evaluation], APIResponse[Agent | Dataset]],
        )

        if include is not omit and include:
            response = embed_included_list(response, id_getter=lambda evaluation: evaluation.id)

        return self._unwrap(response)

    def delete(
        self,
        evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete an evaluation by its ID.

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to delete.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        response = self._delete(
            f"/v2/evaluations/{evaluation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def bulk_delete(
        self,
        *,
        evaluation_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple evaluations at once.

        Parameters
        ----------
        evaluation_ids : sequence of str
            The IDs of the evaluations to delete.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        None
        """
        response = self._delete(
            "/v2/evaluations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"evaluation_ids": evaluation_ids}, EvaluationBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    def create_local(
        self,
        *,
        agent_info: MinimalAgentParam,
        name: Optional[str] | Omit = omit,
        dataset_id: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        old_evaluation_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Evaluation:
        """Create a local evaluation for running agent checks without a remote agent.

        Parameters
        ----------
        agent_info : MinimalAgentParam
            Minimal agent information (name and optional description) to use
            for the evaluation.
        name : str, optional
            The name of the evaluation.
        dataset_id : str, optional
            The ID of the dataset to draw test cases from. Exactly one of
            ``dataset_id`` or ``old_evaluation_id`` must be provided.
        tags : sequence of str, optional
            Optional tags to restrict the subset to test cases carrying those
            tags. Only used when ``dataset_id`` is provided.
        old_evaluation_id : str, optional
            The ID of a previous evaluation whose test cases should be reused.
            Exactly one of ``old_evaluation_id`` or ``dataset_id`` must be
            provided.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        Evaluation
            The newly created local evaluation.

        Raises
        ------
        ValueError
            If neither or both of ``dataset_id`` and ``old_evaluation_id`` are
            provided.
        """
        _validate_dataset_or_old_evaluation(dataset_id, old_evaluation_id)
        criteria: DatasetSubsetParam | CriterionEvaluationDataset | Omit = omit

        if dataset_id is not omit:
            criteria = DatasetSubsetParam(
                dataset_id=cast(str, dataset_id),
                tags=None if tags is omit else cast(Optional[SequenceNotStr[str]], tags),
            )
        elif old_evaluation_id is not omit:
            criteria = CriterionEvaluationDataset(evaluation_id=cast(str, old_evaluation_id))

        response = self._post(
            "/v2/evaluations/create-local",
            body=maybe_transform(
                {
                    "criteria": [criteria],
                    "model": agent_info,
                    "name": name,
                },
                EvaluationCreateLocalParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Evaluation],
        )

        return self._unwrap(response)

    def rerun_errored_results(
        self,
        evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Evaluation:
        """Rerun all evaluation results that ended in an error state.

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to rerun errored results for.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        Evaluation
            The evaluation after rerunning errored results.

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        response = self._post(
            f"/v2/evaluations/{evaluation_id}/rerun-errored-results",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Evaluation],
        )

        return self._unwrap(response)

    def run_single(
        self,
        *,
        checks: Iterable[CheckConfigParam],
        messages: Iterable[ChatMessageParam],
        agent_output: AgentOutputParam,
        agent_description: str | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[CheckResult]:
        """Run a single check against a provided agent output without creating a full evaluation.

        Parameters
        ----------
        checks : iterable of CheckConfigParam
            The checks to run for the evaluation.
        messages : iterable of ChatMessageParam
            The messages to send to the agent.
        agent_output : AgentOutputParam
            The output from the agent.
        agent_description : str or Omit
            The description of the agent.
        project_id : str, optional
            The ID of the project to run the evaluation for.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        list of CheckResult
            The results of the checks.
        """
        api_checks: Iterable[dict[str, object]] = _check_params_to_api(checks)

        response = self._post(
            "/v2/evaluations/run-single",
            body=maybe_transform(
                {
                    "checks": api_checks,
                    "messages": messages,
                    "model_output": agent_output,
                    "model_description": agent_description,
                    "project_id": project_id,
                },
                EvaluationRunSingleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[CheckResult]],
        )

        return self._unwrap(response)


class AsyncEvaluationsResource(AsyncAPIResource):
    @cached_property
    def results(self) -> AsyncResultsResource:
        return AsyncResultsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncEvaluationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEvaluationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEvaluationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncEvaluationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        project_id: str,
        agent_id: str,
        name: str | Omit = omit,
        dataset_id: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        old_evaluation_id: Optional[str] | Omit = omit,
        run_count: int | Omit = omit,
        scheduled_evaluation_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Evaluation:
        """Create and launch a new evaluation of an agent on a dataset.

        Parameters
        ----------
        project_id : str
            The ID of the project to create the evaluation for.
        agent_id : str
            The ID of the agent to create the evaluation for.
        name : str or Omit
            The name of the evaluation.
        dataset_id : str, optional
            The ID of the dataset to draw test cases from. Exactly one of
            ``dataset_id`` or ``old_evaluation_id`` must be provided.
        tags : sequence of str, optional
            Optional tags to restrict the subset to test cases carrying those
            tags. Only used when ``dataset_id`` is provided.
        old_evaluation_id : str, optional
            The ID of a previous evaluation whose test cases should be reused.
            Exactly one of ``old_evaluation_id`` or ``dataset_id`` must be
            provided.
        run_count : int or Omit
            The number of times to run each test case.
        scheduled_evaluation_id : str, optional
            The ID of the scheduled evaluation to associate with.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        Evaluation
            The newly created evaluation.

        Raises
        ------
        ValueError
            If neither or both of ``dataset_id`` and ``old_evaluation_id`` are
            provided.
        """

        _validate_dataset_or_old_evaluation(dataset_id, old_evaluation_id)
        criteria: DatasetSubsetParam | CriterionEvaluationDataset | Omit = omit

        if dataset_id is not omit:
            criteria = DatasetSubsetParam(
                dataset_id=cast(str, dataset_id),
                tags=None if tags is omit else cast(Optional[SequenceNotStr[str]], tags),
            )

        response = await self._post(
            "/v2/evaluations",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "project_id": project_id,
                    "criteria": criteria,
                    "name": name,
                    "old_evaluation_id": old_evaluation_id,
                    "run_count": run_count,
                    "scheduled_evaluation_id": scheduled_evaluation_id,
                },
                EvaluationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Evaluation],
        )

        return self._unwrap(response)

    async def retrieve(
        self,
        evaluation_id: str,
        *,
        include: Optional[List[Literal["agent", "dataset"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Evaluation:
        """Retrieve an evaluation by its ID, with optional related resource inclusion.

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to retrieve.
        include : list of {"agent", "dataset"}, optional
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        Evaluation
            The requested evaluation.

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")

        response = await self._get(
            f"/v2/evaluations/{evaluation_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"include": include}, EvaluationRetrieveParams),
            ),
            cast_to=APIResponseWithIncluded[Evaluation, APIResponse[Agent | Dataset]],
        )

        if include is not omit and include:
            response = embed_included_single(response, id_getter=lambda evaluation: evaluation.id)

        return self._unwrap(response)

    async def update(
        self,
        evaluation_id: str,
        *,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Evaluation:
        """Update an evaluation's name.

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to update.
        name : str
            The name of the evaluation.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        Evaluation
            The updated evaluation.

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        response = await self._patch(
            f"/v2/evaluations/{evaluation_id}",
            body=await async_maybe_transform({"name": name}, EvaluationUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Evaluation],
        )

        return self._unwrap(response)

    async def list(
        self,
        *,
        project_id: str,
        include: Optional[List[Literal["agent", "dataset"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[Evaluation]:
        """List all evaluations for a project, with optional related resource inclusion.

        Parameters
        ----------
        project_id : str
            The ID of the project to list evaluations for.
        include : list of {"agent", "dataset"}, optional
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        list of Evaluation
            The evaluations belonging to the project.
        """
        response = await self._get(
            "/v2/evaluations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "project_id": project_id,
                        "include": include,
                    },
                    EvaluationListParams,
                ),
            ),
            cast_to=APIResponseWithIncluded[List[Evaluation], APIResponse[Agent | Dataset]],
        )

        if include is not omit and include:
            response = embed_included_list(response, id_getter=lambda evaluation: evaluation.id)

        return self._unwrap(response)

    async def delete(
        self,
        evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete an evaluation by its ID.

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to delete.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        None

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        response = await self._delete(
            f"/v2/evaluations/{evaluation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def bulk_delete(
        self,
        *,
        evaluation_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete multiple evaluations at once.

        Parameters
        ----------
        evaluation_ids : sequence of str
            The IDs of the evaluations to delete.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        None
        """
        response = await self._delete(
            "/v2/evaluations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"evaluation_ids": evaluation_ids}, EvaluationBulkDeleteParams),
            ),
            cast_to=APIResponse[None],
        )

        return self._unwrap(response)

    async def create_local(
        self,
        *,
        agent_info: MinimalAgentParam,
        name: Optional[str] | Omit = omit,
        dataset_id: Optional[str] | Omit = omit,
        tags: Optional[SequenceNotStr[str]] | Omit = omit,
        old_evaluation_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Evaluation:
        """Create a local evaluation for running agent checks without a remote agent.

        Parameters
        ----------
        agent_info : MinimalAgentParam
            Minimal agent information (name and optional description) to use
            for the evaluation.
        name : str, optional
            The name of the evaluation.
        dataset_id : str, optional
            The ID of the dataset to draw test cases from. Exactly one of
            ``dataset_id`` or ``old_evaluation_id`` must be provided.
        tags : sequence of str, optional
            Optional tags to restrict the subset to test cases carrying those
            tags. Only used when ``dataset_id`` is provided.
        old_evaluation_id : str, optional
            The ID of a previous evaluation whose test cases should be reused.
            Exactly one of ``old_evaluation_id`` or ``dataset_id`` must be
            provided.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        Evaluation
            The newly created local evaluation.

        Raises
        ------
        ValueError
            If neither or both of ``dataset_id`` and ``old_evaluation_id`` are
            provided.
        """
        _validate_dataset_or_old_evaluation(dataset_id, old_evaluation_id)
        criteria: DatasetSubsetParam | CriterionEvaluationDataset | Omit = omit

        if dataset_id is not omit:
            criteria = DatasetSubsetParam(
                dataset_id=cast(str, dataset_id),
                tags=None if tags is omit else cast(Optional[SequenceNotStr[str]], tags),
            )
        elif old_evaluation_id is not omit:
            criteria = CriterionEvaluationDataset(evaluation_id=cast(str, old_evaluation_id))

        response = await self._post(
            "/v2/evaluations/create-local",
            body=await async_maybe_transform(
                {
                    "criteria": [criteria],
                    "model": agent_info,
                    "name": name,
                },
                EvaluationCreateLocalParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Evaluation],
        )

        return self._unwrap(response)

    async def rerun_errored_results(
        self,
        evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Evaluation:
        """Rerun all evaluation results that ended in an error state.

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to rerun errored results for.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        Evaluation
            The evaluation after rerunning errored results.

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        response = await self._post(
            f"/v2/evaluations/{evaluation_id}/rerun-errored-results",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[Evaluation],
        )

        return self._unwrap(response)

    async def run_single(
        self,
        *,
        checks: Iterable[CheckConfigParam],
        messages: Iterable[ChatMessageParam],
        agent_output: AgentOutputParam,
        agent_description: str | Omit = omit,
        project_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[CheckResult]:
        """Run a single check against a provided agent output without creating a full evaluation.

        Parameters
        ----------
        checks : iterable of CheckConfigParam
            The checks to run for the evaluation.
        messages : iterable of ChatMessageParam
            The messages to send to the agent.
        agent_output : AgentOutputParam
            The output from the agent.
        agent_description : str or Omit
            The description of the agent.
        project_id : str, optional
            The ID of the project to run the evaluation for.

        Other Parameters
        ----------------
        extra_headers : Headers or None
            Send extra headers.
        extra_query : Query or None
            Add additional query parameters to the request.
        extra_body : Body or None
            Add additional JSON properties to the request.
        timeout : float, httpx.Timeout, None, or NotGiven
            Override the client-level default timeout for this request, in
            seconds.

        Returns
        -------
        list of CheckResult
            The results of the checks.
        """
        api_checks: Iterable[dict[str, object]] = _check_params_to_api(checks)

        response = await self._post(
            "/v2/evaluations/run-single",
            body=await async_maybe_transform(
                {
                    "checks": api_checks,
                    "messages": messages,
                    "model_output": agent_output,
                    "model_description": agent_description,
                    "project_id": project_id,
                },
                EvaluationRunSingleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[List[CheckResult]],
        )

        return self._unwrap(response)


class EvaluationsResourceWithRawResponse:
    def __init__(self, evaluations: EvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = to_raw_response_wrapper(
            evaluations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            evaluations.retrieve,
        )
        self.update = to_raw_response_wrapper(
            evaluations.update,
        )
        self.list = to_raw_response_wrapper(
            evaluations.list,
        )
        self.delete = to_raw_response_wrapper(
            evaluations.delete,
        )
        self.bulk_delete = to_raw_response_wrapper(
            evaluations.bulk_delete,
        )
        self.create_local = to_raw_response_wrapper(
            evaluations.create_local,
        )
        self.rerun_errored_results = to_raw_response_wrapper(
            evaluations.rerun_errored_results,
        )
        self.run_single = to_raw_response_wrapper(
            evaluations.run_single,
        )

    @cached_property
    def results(self) -> ResultsResourceWithRawResponse:
        return ResultsResourceWithRawResponse(self._evaluations.results)


class AsyncEvaluationsResourceWithRawResponse:
    def __init__(self, evaluations: AsyncEvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = async_to_raw_response_wrapper(
            evaluations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            evaluations.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            evaluations.update,
        )
        self.list = async_to_raw_response_wrapper(
            evaluations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            evaluations.delete,
        )
        self.bulk_delete = async_to_raw_response_wrapper(
            evaluations.bulk_delete,
        )
        self.create_local = async_to_raw_response_wrapper(
            evaluations.create_local,
        )
        self.rerun_errored_results = async_to_raw_response_wrapper(
            evaluations.rerun_errored_results,
        )
        self.run_single = async_to_raw_response_wrapper(
            evaluations.run_single,
        )

    @cached_property
    def results(self) -> AsyncResultsResourceWithRawResponse:
        return AsyncResultsResourceWithRawResponse(self._evaluations.results)


class EvaluationsResourceWithStreamingResponse:
    def __init__(self, evaluations: EvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = to_streamed_response_wrapper(
            evaluations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            evaluations.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            evaluations.update,
        )
        self.list = to_streamed_response_wrapper(
            evaluations.list,
        )
        self.delete = to_streamed_response_wrapper(
            evaluations.delete,
        )
        self.bulk_delete = to_streamed_response_wrapper(
            evaluations.bulk_delete,
        )
        self.create_local = to_streamed_response_wrapper(
            evaluations.create_local,
        )
        self.rerun_errored_results = to_streamed_response_wrapper(
            evaluations.rerun_errored_results,
        )
        self.run_single = to_streamed_response_wrapper(
            evaluations.run_single,
        )

    @cached_property
    def results(self) -> ResultsResourceWithStreamingResponse:
        return ResultsResourceWithStreamingResponse(self._evaluations.results)


class AsyncEvaluationsResourceWithStreamingResponse:
    def __init__(self, evaluations: AsyncEvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = async_to_streamed_response_wrapper(
            evaluations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            evaluations.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            evaluations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            evaluations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            evaluations.delete,
        )
        self.bulk_delete = async_to_streamed_response_wrapper(
            evaluations.bulk_delete,
        )
        self.create_local = async_to_streamed_response_wrapper(
            evaluations.create_local,
        )
        self.rerun_errored_results = async_to_streamed_response_wrapper(
            evaluations.rerun_errored_results,
        )
        self.run_single = async_to_streamed_response_wrapper(
            evaluations.run_single,
        )

    @cached_property
    def results(self) -> AsyncResultsResourceWithStreamingResponse:
        return AsyncResultsResourceWithStreamingResponse(self._evaluations.results)



from __future__ import annotations

from typing import List, Tuple, Literal, Optional, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .._included import embed_included_single, embed_included_paginated
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.agent import AgentOutputParam
from ..._base_client import make_request_options
from ...types.common import APIResponse, APIPaginatedMetadata, APIPaginatedResponse, APIResponseWithIncluded
from ...types.test_case import TestCase
from ...types.evaluation import (
    ResultFiltersParam,
    ResultOrderByParam,
    ResultSearchParams,
    ResultUpdateParams,
    TestCaseEvaluation,
    FailureCategoryParam,
    ResultRetrieveParams,
    ResultUpdateVisibilityParams,
    ResultSubmitLocalOutputParams,
)

__all__ = ["ResultsResource", "AsyncResultsResource"]


class ResultsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ResultsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return ResultsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResultsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return ResultsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        result_id: str,
        *,
        evaluation_id: str,
        include: Optional[List[Literal["test_case"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseEvaluation:
        """Retrieve a specific evaluation result by its ID.

        Parameters
        ----------
        result_id : str
            The ID of the result to retrieve.
        evaluation_id : str
            The ID of the evaluation to retrieve the result for.
        include : Optional[List[Literal["test_case"]]], optional
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseEvaluation
            The retrieved evaluation result.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")

        if not result_id:
            raise ValueError(f"Expected a non-empty value for `result_id` but received {result_id!r}")

        response = self._get(
            f"/v2/evaluations/{evaluation_id}/results/{result_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include": include}, ResultRetrieveParams),
            ),
            cast_to=APIResponseWithIncluded[TestCaseEvaluation, APIResponse[TestCase]],
        )

        if include is not omit and include:
            response = embed_included_single(response, id_getter=lambda result: result.id)

        return self._unwrap(response)

    def update(
        self,
        result_id: str,
        *,
        evaluation_id: str,
        failure_category: Optional[FailureCategoryParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseEvaluation:
        """Update the failure category of an evaluation result.

        Parameters
        ----------
        result_id : str
            The ID of the result to update.
        evaluation_id : str
            The ID of the evaluation to update the result for.
        failure_category : Optional[FailureCategoryParam], optional
            The failure category to update the result for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseEvaluation
            The updated evaluation result.

        Raises
        ------
        ValueError
            If ``evaluation_id`` or ``result_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not result_id:
            raise ValueError(f"Expected a non-empty value for `result_id` but received {result_id!r}")
        response = self._patch(
            f"/v2/evaluations/{evaluation_id}/results/{result_id}",
            body=maybe_transform({"failure_category": failure_category}, ResultUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseEvaluation],
        )

        return self._unwrap(response)

    def rerun_test_case(
        self,
        result_id: str,
        *,
        evaluation_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseEvaluation:
        """Rerun a single test case evaluation to get fresh results.

        Parameters
        ----------
        result_id : str
            The ID of the result to rerun the test case for.
        evaluation_id : str
            The ID of the evaluation to rerun the test case for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseEvaluation
            The rerun evaluation result.

        Raises
        ------
        ValueError
            If ``evaluation_id`` or ``result_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not result_id:
            raise ValueError(f"Expected a non-empty value for `result_id` but received {result_id!r}")
        response = self._post(
            f"/v2/evaluations/{evaluation_id}/results/{result_id}/rerun-test-case",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseEvaluation],
        )

        return self._unwrap(response)

    def submit_local_output(
        self,
        result_id: str,
        *,
        evaluation_id: str,
        error: Optional[str] | Omit = omit,
        agent_output: Optional[AgentOutputParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseEvaluation:
        """Submit a locally-generated agent output for evaluation and scoring.

        Parameters
        ----------
        result_id : str
            The ID of the result to submit the local output for.
        evaluation_id : str
            The ID of the evaluation to submit the local output for.
        error : Optional[str], optional
            The error to submit the local output for.
        agent_output : Optional[AgentOutputParam], optional
            The output to submit the local output for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseEvaluation
            The evaluation result with the submitted output.

        Raises
        ------
        ValueError
            If ``evaluation_id`` or ``result_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not result_id:
            raise ValueError(f"Expected a non-empty value for `result_id` but received {result_id!r}")
        response = self._post(
            f"/v2/evaluations/{evaluation_id}/results/{result_id}/submit-local-output",
            body=maybe_transform(
                {
                    "error": error,
                    "output": agent_output,
                },
                ResultSubmitLocalOutputParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseEvaluation],
        )

        return self._unwrap(response)

    def list(
        self,
        evaluation_id: str,
        *,
        include: Optional[List[Literal["test_case"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[TestCaseEvaluation]:
        """List all results for a given evaluation.

        Fetches every page via :meth:`search` (same as an unfiltered search).

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to list the results for.
        include : Optional[List[Literal["test_case"]]], optional
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[TestCaseEvaluation]
            A list of evaluation results.

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")

        page_limit = 100
        all_items: List[TestCaseEvaluation] = []
        offset = 0
        while True:
            page, meta = self.search(
                evaluation_id,
                limit=page_limit,
                offset=offset,
                include=include,
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                include_metadata=True,
            )
            all_items.extend(page)
            next_offset = meta.offset + meta.count
            if next_offset >= meta.total or not page:
                break
            offset = next_offset
        return all_items

    @overload
    def search(
        self,
        evaluation_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[ResultOrderByParam]] | Omit = omit,
        filters: Optional[ResultFiltersParam] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        offset: Optional[int] | Omit = omit,
        include: Optional[List[Literal["test_case"]]] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[False] = False,
    ) -> List[TestCaseEvaluation]: ...

    @overload
    def search(
        self,
        evaluation_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[ResultOrderByParam]] | Omit = omit,
        filters: Optional[ResultFiltersParam] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        offset: Optional[int] | Omit = omit,
        include: Optional[List[Literal["test_case"]]] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[True],
    ) -> Tuple[List[TestCaseEvaluation], APIPaginatedMetadata]: ...

    def search(
        self,
        evaluation_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[ResultOrderByParam]] | Omit = omit,
        filters: Optional[ResultFiltersParam] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        offset: Optional[int] | Omit = omit,
        include: Optional[List[Literal["test_case"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[TestCaseEvaluation] | Tuple[List[TestCaseEvaluation], APIPaginatedMetadata]:
        """Search evaluation results using filters, sorting, and pagination.

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to search the results for.
        query : Optional[str], optional
            Search query for evaluation results.
        order_by : Optional[List[ResultOrderByParam]], optional
            Order by criteria for evaluation results.
        filters : Optional[ResultFiltersParam], optional
            Filter criteria for evaluation results.
        limit : Optional[int], optional
            Maximum number of results to return.
        offset : Optional[int], optional
            Number of results to skip.
        include : Optional[List[Literal["test_case"]]], optional
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[TestCaseEvaluation] | Tuple[List[TestCaseEvaluation], APIPaginatedMetadata]
            The search results, optionally with pagination metadata.

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")

        response = self._post(
            f"/v2/evaluations/{evaluation_id}/results/search",
            body=maybe_transform({"filters": filters, "order_by": order_by, "search": query}, ResultSearchParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "include": include,
                    },
                    ResultSearchParams,
                ),
            ),
            cast_to=APIPaginatedResponse[TestCaseEvaluation, APIResponse[TestCase]],
        )

        if include is not omit and include:
            response = embed_included_paginated(response, id_getter=lambda result: result.id)

        return self._unwrap_paginated(response, include_metadata)

    def update_visibility(
        self,
        result_id: str,
        *,
        evaluation_id: str,
        hidden: bool,
        set_test_case_draft: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseEvaluation:
        """Update the visibility of an evaluation result, optionally setting the linked test case to draft.

        Parameters
        ----------
        result_id : str
            The ID of the result to update the visibility for.
        evaluation_id : str
            The ID of the evaluation to update the visibility for.
        hidden : bool
            Whether the result should be hidden.
        set_test_case_draft : Optional[bool], optional
            Whether the test case should be set to draft.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseEvaluation
            The updated evaluation result.

        Raises
        ------
        ValueError
            If ``evaluation_id`` or ``result_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not result_id:
            raise ValueError(f"Expected a non-empty value for `result_id` but received {result_id!r}")
        response = self._patch(
            f"/v2/evaluations/{evaluation_id}/results/{result_id}/visibility",
            body=maybe_transform(
                {"hidden": hidden, "set_test_case_draft": set_test_case_draft},
                ResultUpdateVisibilityParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseEvaluation],
        )

        return self._unwrap(response)


class AsyncResultsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncResultsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#accessing-raw-response-data-eg-headers
        """
        return AsyncResultsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResultsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Giskard-AI/giskard-hub-python#with_streaming_response
        """
        return AsyncResultsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        result_id: str,
        *,
        evaluation_id: str,
        include: Optional[List[Literal["test_case"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseEvaluation:
        """Retrieve a specific evaluation result by its ID.

        Parameters
        ----------
        result_id : str
            The ID of the result to retrieve.
        evaluation_id : str
            The ID of the evaluation to retrieve the result for.
        include : Optional[List[Literal["test_case"]]], optional
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseEvaluation
            The retrieved evaluation result.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")

        if not result_id:
            raise ValueError(f"Expected a non-empty value for `result_id` but received {result_id!r}")

        response = await self._get(
            f"/v2/evaluations/{evaluation_id}/results/{result_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"include": include}, ResultRetrieveParams),
            ),
            cast_to=APIResponseWithIncluded[TestCaseEvaluation, APIResponse[TestCase]],
        )

        if include is not omit and include:
            response = embed_included_single(response, id_getter=lambda result: result.id)

        return self._unwrap(response)

    async def update(
        self,
        result_id: str,
        *,
        evaluation_id: str,
        failure_category: Optional[FailureCategoryParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseEvaluation:
        """Update the failure category of an evaluation result.

        Parameters
        ----------
        result_id : str
            The ID of the result to update.
        evaluation_id : str
            The ID of the evaluation to update the result for.
        failure_category : Optional[FailureCategoryParam], optional
            The failure category to update the result for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseEvaluation
            The updated evaluation result.

        Raises
        ------
        ValueError
            If ``evaluation_id`` or ``result_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not result_id:
            raise ValueError(f"Expected a non-empty value for `result_id` but received {result_id!r}")
        response = await self._patch(
            f"/v2/evaluations/{evaluation_id}/results/{result_id}",
            body=await async_maybe_transform({"failure_category": failure_category}, ResultUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseEvaluation],
        )

        return self._unwrap(response)

    async def rerun_test_case(
        self,
        result_id: str,
        *,
        evaluation_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseEvaluation:
        """Rerun a single test case evaluation to get fresh results.

        Parameters
        ----------
        result_id : str
            The ID of the result to rerun the test case for.
        evaluation_id : str
            The ID of the evaluation to rerun the test case for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseEvaluation
            The rerun evaluation result.

        Raises
        ------
        ValueError
            If ``evaluation_id`` or ``result_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not result_id:
            raise ValueError(f"Expected a non-empty value for `result_id` but received {result_id!r}")
        response = await self._post(
            f"/v2/evaluations/{evaluation_id}/results/{result_id}/rerun-test-case",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseEvaluation],
        )

        return self._unwrap(response)

    async def submit_local_output(
        self,
        result_id: str,
        *,
        evaluation_id: str,
        error: Optional[str] | Omit = omit,
        agent_output: Optional[AgentOutputParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseEvaluation:
        """Submit a locally-generated agent output for evaluation and scoring.

        Parameters
        ----------
        result_id : str
            The ID of the result to submit the local output for.
        evaluation_id : str
            The ID of the evaluation to submit the local output for.
        error : Optional[str], optional
            The error to submit the local output for.
        agent_output : Optional[AgentOutputParam], optional
            The output to submit the local output for.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseEvaluation
            The evaluation result with the submitted output.

        Raises
        ------
        ValueError
            If ``evaluation_id`` or ``result_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not result_id:
            raise ValueError(f"Expected a non-empty value for `result_id` but received {result_id!r}")
        response = await self._post(
            f"/v2/evaluations/{evaluation_id}/results/{result_id}/submit-local-output",
            body=await async_maybe_transform(
                {
                    "error": error,
                    "output": agent_output,
                },
                ResultSubmitLocalOutputParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseEvaluation],
        )

        return self._unwrap(response)

    async def list(
        self,
        evaluation_id: str,
        *,
        include: Optional[List[Literal["test_case"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> List[TestCaseEvaluation]:
        """List all results for a given evaluation.

        Fetches every page via :meth:`search` (same as an unfiltered search).

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to list the results for.
        include : Optional[List[Literal["test_case"]]], optional
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[TestCaseEvaluation]
            A list of evaluation results.

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")

        page_limit = 100
        all_items: List[TestCaseEvaluation] = []
        offset = 0
        while True:
            page, meta = await self.search(
                evaluation_id,
                limit=page_limit,
                offset=offset,
                include=include,
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                include_metadata=True,
            )
            all_items.extend(page)
            next_offset = meta.offset + meta.count
            if next_offset >= meta.total or not page:
                break
            offset = next_offset
        return all_items

    @overload
    async def search(
        self,
        evaluation_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[ResultOrderByParam]] | Omit = omit,
        filters: Optional[ResultFiltersParam] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        offset: Optional[int] | Omit = omit,
        include: Optional[List[Literal["test_case"]]] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[False] = False,
    ) -> List[TestCaseEvaluation]: ...

    @overload
    async def search(
        self,
        evaluation_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[ResultOrderByParam]] | Omit = omit,
        filters: Optional[ResultFiltersParam] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        offset: Optional[int] | Omit = omit,
        include: Optional[List[Literal["test_case"]]] | Omit = omit,
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: Literal[True],
    ) -> Tuple[List[TestCaseEvaluation], APIPaginatedMetadata]: ...

    async def search(
        self,
        evaluation_id: str,
        *,
        query: Optional[str] | Omit = omit,
        order_by: Optional[List[ResultOrderByParam]] | Omit = omit,
        filters: Optional[ResultFiltersParam] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        offset: Optional[int] | Omit = omit,
        include: Optional[List[Literal["test_case"]]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        include_metadata: bool = False,
    ) -> List[TestCaseEvaluation] | Tuple[List[TestCaseEvaluation], APIPaginatedMetadata]:
        """Search evaluation results using filters, sorting, and pagination.

        Parameters
        ----------
        evaluation_id : str
            The ID of the evaluation to search the results for.
        query : Optional[str], optional
            Search query for evaluation results.
        order_by : Optional[List[ResultOrderByParam]], optional
            Order by criteria for evaluation results.
        filters : Optional[ResultFiltersParam], optional
            Filter criteria for evaluation results.
        limit : Optional[int], optional
            Maximum number of results to return.
        offset : Optional[int], optional
            Number of results to skip.
        include : Optional[List[Literal["test_case"]]], optional
            Related resources to include in response.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        List[TestCaseEvaluation] | Tuple[List[TestCaseEvaluation], APIPaginatedMetadata]
            The search results, optionally with pagination metadata.

        Raises
        ------
        ValueError
            If ``evaluation_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")

        response = await self._post(
            f"/v2/evaluations/{evaluation_id}/results/search",
            body=await async_maybe_transform(
                {"filters": filters, "order_by": order_by, "search": query}, ResultSearchParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "include": include,
                    },
                    ResultSearchParams,
                ),
            ),
            cast_to=APIPaginatedResponse[TestCaseEvaluation, APIResponse[TestCase]],
        )

        if include is not omit and include:
            response = embed_included_paginated(response, id_getter=lambda result: result.id)

        return self._unwrap_paginated(response, include_metadata)

    async def update_visibility(
        self,
        result_id: str,
        *,
        evaluation_id: str,
        hidden: bool,
        set_test_case_draft: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TestCaseEvaluation:
        """Update the visibility of an evaluation result, optionally setting the linked test case to draft.

        Parameters
        ----------
        result_id : str
            The ID of the result to update the visibility for.
        evaluation_id : str
            The ID of the evaluation to update the visibility for.
        hidden : bool
            Whether the result should be hidden.
        set_test_case_draft : Optional[bool], optional
            Whether the test case should be set to draft.

        Other Parameters
        ----------------
        extra_headers : Headers | None
            Send extra headers.
        extra_query : Query | None
            Add additional query parameters to the request.
        extra_body : Body | None
            Add additional JSON properties to the request.
        timeout : float | httpx.Timeout | None | NotGiven
            Override the client-level default timeout for this request, in seconds.

        Returns
        -------
        TestCaseEvaluation
            The updated evaluation result.

        Raises
        ------
        ValueError
            If ``evaluation_id`` or ``result_id`` is empty.
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        if not result_id:
            raise ValueError(f"Expected a non-empty value for `result_id` but received {result_id!r}")
        response = await self._patch(
            f"/v2/evaluations/{evaluation_id}/results/{result_id}/visibility",
            body=await async_maybe_transform(
                {"hidden": hidden, "set_test_case_draft": set_test_case_draft},
                ResultUpdateVisibilityParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=APIResponse[TestCaseEvaluation],
        )

        return self._unwrap(response)


class ResultsResourceWithRawResponse:
    def __init__(self, results: ResultsResource) -> None:
        self._results = results

        self.retrieve = to_raw_response_wrapper(
            results.retrieve,
        )
        self.update = to_raw_response_wrapper(
            results.update,
        )
        self.rerun_test_case = to_raw_response_wrapper(
            results.rerun_test_case,
        )
        self.submit_local_output = to_raw_response_wrapper(
            results.submit_local_output,
        )
        self.list = to_raw_response_wrapper(
            results.list,
        )
        self.search = to_raw_response_wrapper(
            results.search,
        )
        self.update_visibility = to_raw_response_wrapper(
            results.update_visibility,
        )


class AsyncResultsResourceWithRawResponse:
    def __init__(self, results: AsyncResultsResource) -> None:
        self._results = results

        self.retrieve = async_to_raw_response_wrapper(
            results.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            results.update,
        )
        self.rerun_test_case = async_to_raw_response_wrapper(
            results.rerun_test_case,
        )
        self.submit_local_output = async_to_raw_response_wrapper(
            results.submit_local_output,
        )
        self.list = async_to_raw_response_wrapper(
            results.list,
        )
        self.search = async_to_raw_response_wrapper(
            results.search,
        )
        self.update_visibility = async_to_raw_response_wrapper(
            results.update_visibility,
        )


class ResultsResourceWithStreamingResponse:
    def __init__(self, results: ResultsResource) -> None:
        self._results = results

        self.retrieve = to_streamed_response_wrapper(
            results.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            results.update,
        )
        self.rerun_test_case = to_streamed_response_wrapper(
            results.rerun_test_case,
        )
        self.submit_local_output = to_streamed_response_wrapper(
            results.submit_local_output,
        )
        self.list = to_streamed_response_wrapper(
            results.list,
        )
        self.search = to_streamed_response_wrapper(
            results.search,
        )
        self.update_visibility = to_streamed_response_wrapper(
            results.update_visibility,
        )


class AsyncResultsResourceWithStreamingResponse:
    def __init__(self, results: AsyncResultsResource) -> None:
        self._results = results

        self.retrieve = async_to_streamed_response_wrapper(
            results.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            results.update,
        )
        self.rerun_test_case = async_to_streamed_response_wrapper(
            results.rerun_test_case,
        )
        self.submit_local_output = async_to_streamed_response_wrapper(
            results.submit_local_output,
        )
        self.list = async_to_streamed_response_wrapper(
            results.list,
        )
        self.search = async_to_streamed_response_wrapper(
            results.search,
        )
        self.update_visibility = async_to_streamed_response_wrapper(
            results.update_visibility,
        )



"""Scheduled evaluation domain types."""

from typing import List, Union, Literal, Optional, Annotated, TypeAlias, TypedDict
from datetime import datetime
from typing_extensions import Required

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .._models import BaseModel
from .execution import (
    ErrorExecutionStatus,
    SuccessExecutionStatus,
    ErrorExecutionStatusParam,
    SuccessExecutionStatusParam,
)
from .evaluation import EvaluationReference

__all__ = [
    "FrequencyOption",
    "LastExecutionStatus",
    "LastExecutionStatusParam",
    "ScheduledEvaluation",
    "ScheduledEvaluationListParams",
    "ScheduledEvaluationCreateParams",
    "ScheduledEvaluationUpdateParams",
    "ScheduledEvaluationRetrieveParams",
    "ScheduledEvaluationBulkDeleteParams",
    "ScheduledEvaluationListEvaluationsParams",
]


# ---------------------------------------------------------------------------
# Enums
# ---------------------------------------------------------------------------

FrequencyOption: TypeAlias = Literal["daily", "weekly", "monthly"]


# ---------------------------------------------------------------------------
# Models
# ---------------------------------------------------------------------------

LastExecutionStatus: TypeAlias = Union[SuccessExecutionStatus, ErrorExecutionStatus, None]


class ScheduledEvaluation(BaseModel):
    id: str
    project_id: str
    created_at: datetime
    updated_at: datetime
    name: str
    agent_id: str
    dataset_id: str
    tags: List[str]
    run_count: int
    frequency: FrequencyOption
    time: str
    day_of_week: Optional[int] = None
    day_of_month: Optional[int] = None
    last_execution_at: Optional[datetime] = None
    last_execution_status: Optional[LastExecutionStatus] = None
    paused: bool
    evaluations: List[EvaluationReference]


# ---------------------------------------------------------------------------
# Params
# ---------------------------------------------------------------------------


class ScheduledEvaluationListParams(TypedDict, total=False):
    project_id: Required[str]
    last_days: Optional[int]
    include: Optional[List[Literal["evaluations"]]]


class ScheduledEvaluationCreateParams(TypedDict, total=False):
    project_id: Required[str]
    name: Required[str]
    agent_id: Required[str]
    dataset_id: Required[str]
    tags: SequenceNotStr[str]
    run_count: int
    frequency: Required[FrequencyOption]
    time: Required[str]
    day_of_week: Optional[int]
    day_of_month: Optional[int]


LastExecutionStatusParam: TypeAlias = Union[SuccessExecutionStatusParam, ErrorExecutionStatusParam]


class ScheduledEvaluationUpdateParams(TypedDict, total=False):
    name: Optional[str]
    run_count: Optional[int]
    frequency: Optional[FrequencyOption]
    time: Optional[str]
    day_of_week: Optional[int]
    day_of_month: Optional[int]
    last_execution_at: Optional[Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]]
    last_execution_status: Optional[LastExecutionStatusParam]
    paused: Optional[bool]


class ScheduledEvaluationRetrieveParams(TypedDict, total=False):
    include: Optional[List[Literal["evaluations"]]]


class ScheduledEvaluationBulkDeleteParams(TypedDict, total=False):
    scheduled_evaluation_ids: Required[SequenceNotStr[str]]


class ScheduledEvaluationListEvaluationsParams(TypedDict, total=False):
    include: Optional[List[Literal["agent", "dataset"]]]
